var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
define(["require", "exports", "./animators/Main", "./Teams"], function (require, exports, Main_1, Teams_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Finds the index of the first alive actor.
     *
     * @param actors   A list of actors to be sent out into battle.
     * @returns Index of the first alive actor.
     */
    var findFirstAliveIndex = function (actors) {
        for (var i = 0; i < actors.length; i += 1) {
            if (actors[i].statistics.health.current !== 0) {
                return i;
            }
        }
        throw new Error("Cannot create team since no actors are alive.");
    };
    /**
     * Drives RPG-like battles between two teams of actors.
     */
    var BattleMovr = /** @class */ (function () {
        /**
         * Initializes a new instance of the BattleMovr class.
         *
         * @param settings   Settings to be used for initialization.
         */
        function BattleMovr(settings) {
            this.actionsOrderer = settings.actionsOrderer;
            this.animations = settings.animations;
            this.selectorFactories = settings.selectorFactories;
        }
        /**
         * Gets whether there is a current battle.
         *
         * @returns Whether there is a current battle.
         */
        BattleMovr.prototype.inBattle = function () {
            return this.battleInfo !== undefined;
        };
        /**
         * Gets battle info for the current battle.
         *
         * @returns Battle info for the current battle.
         */
        BattleMovr.prototype.getBattleInfo = function () {
            if (this.battleInfo === undefined) {
                throw new Error("There is no current battle.");
            }
            return this.battleInfo;
        };
        /**
         * Starts a new battle.
         *
         * @param options   Options to start the battle.
         * @returns Battle info for the new battle.
         */
        BattleMovr.prototype.startBattle = function (options) {
            if (this.battleInfo !== undefined) {
                throw new Error("A battle is already happening.");
            }
            this.battleInfo = __assign(__assign({}, options), { choices: {}, teams: {
                    opponent: this.createTeamFromInfo(options.teams.opponent),
                    player: this.createTeamFromInfo(options.teams.player),
                } });
            this.animator = new Main_1.Main({
                animations: this.animations,
                battleInfo: this.battleInfo,
            }, this.actionsOrderer);
            this.animator.run();
            return this.battleInfo;
        };
        /**
         * Switches the selected actor for a team.
         *
         * @param team   Team switching actors.
         * @param newActor   New selected actor for the team.
         */
        BattleMovr.prototype.switchSelectedActor = function (team, newActor) {
            if (this.battleInfo === undefined) {
                throw new Error("No battle is happening.");
            }
            var battleTeam = this.battleInfo.teams[Teams_1.Team[team]];
            var oldActor = battleTeam.selectedActor;
            if (oldActor === newActor) {
                throw new Error("Cannot switch to the currently selected actor.");
            }
            battleTeam.actors[battleTeam.actors.indexOf(newActor)] = oldActor;
            battleTeam.actors[battleTeam.selectedIndex] = newActor;
            battleTeam.selectedActor = newActor;
        };
        /**
         * Stops the current battle.
         *
         * @param outcome   Why the battle stopped.
         * @param onComplete   Callback for when this is over.
         */
        BattleMovr.prototype.stopBattle = function (outcome, onComplete) {
            var _this = this;
            if (this.battleInfo === undefined) {
                throw new Error("No battle is happening.");
            }
            this.animations.complete(outcome, function () {
                _this.animator = undefined;
                _this.battleInfo = undefined;
                if (onComplete !== undefined) {
                    onComplete();
                }
            });
        };
        /**
         * Creates a battle team from starting info.
         *
         * @param team   Starting info on a team.
         * @returns A battle team for the starting info.
         */
        BattleMovr.prototype.createTeamFromInfo = function (team) {
            var selectorFactory = this.selectorFactories[team.selector];
            if (selectorFactory === undefined) {
                throw new Error("Unknown selector type: '" + team.selector + ".");
            }
            var firstAliveIndex = findFirstAliveIndex(team.actors);
            return __assign(__assign({}, team), { orderedActors: team.actors.slice(), selectedActor: team.actors[firstAliveIndex], selectedIndex: firstAliveIndex, selector: selectorFactory() });
        };
        return BattleMovr;
    }());
    exports.BattleMovr = BattleMovr;
});
